namespace PolClient
{
	using System;
	using System.Collections;
	using System.Data;
	using System.Drawing;
	using System.Web;
	using System.Web.UI.WebControls;
	using System.Web.UI.HtmlControls;
	using PageBoxGrid;

	/// <summary>
	/// Shopping basket control.
	/// Methods created or modified:<ul>
	/// <li>Page_Load</li>
	/// <li>OnArticleChanged</li>
	/// <li>OnAdd</li>
	/// <li>OnDelete</li>
	/// <li>Prepare</li>
	/// <li>Commit</li></ul>
	/// <p>Copyright (c) 2002 Alexis Grandemange<br/>
	/// Mail: alexis.grandemange@pagebox.net</p>
	/// <pre>This program is free software; you can redistribute it and/or
	/// modify it under the terms of the GNU Lesser General Public
	/// License as published by the Free Software Foundation; version
	/// 2.1 of the License.
	/// This library is distributed in the hope that it will be useful,
	/// but WITHOUT ANY WARRANTY; without even the implied warranty of
	/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	/// GNU Lesser General Public License for more details.
	/// A copy of the GNU Lesser General Public License lesser.txt should be
	/// included in the distribution.</pre>
	/// </summary>
	public abstract class ShoppingBasket : System.Web.UI.UserControl, PageBoxGrid.Transaction
	{
		protected System.Web.UI.WebControls.Label title;
		protected System.Web.UI.WebControls.Label article;
		protected System.Web.UI.WebControls.DropDownList ddArticle;
		protected System.Web.UI.WebControls.Button ButAdd;
		protected System.Web.UI.WebControls.Label quantity;
		protected System.Web.UI.WebControls.TextBox TxtQty;
		protected System.Web.UI.WebControls.Label labStatus;
		protected System.Web.UI.WebControls.Label copyright;
		protected System.Web.UI.WebControls.DataGrid dgShopping;
		protected System.Web.UI.WebControls.Label price;
		protected System.Web.UI.WebControls.Panel ShoppingPanel;

		/// <summary>
		/// CSS class to use for the title
		/// </summary>
		private string titleCss = null;
		public string TitleCss 
		{
			get { return titleCss; }
			set { titleCss = value; }
		}
		/// <summary>
		/// CSS class to use for the status
		/// </summary>
		private string statusCss = null;
		public string StatusCss 
		{
			get { return statusCss; }
			set { statusCss = value; }
		}
		/// <summary>
		/// CSS class to use for the copyright
		/// </summary>
		private string copyrightCss = null;
		public string CopyrightCss 
		{
			get { return copyrightCss; }
			set { copyrightCss = value; }
		}
		/// <summary>
		/// Table min width (when the Datagrid is not displayed)
		/// </summary>
		private string minWidth = null;
		public string MinWidth 
		{
			get { return minWidth; }
			set { minWidth = value; }
		}
		/// <summary>
		/// Table max width (when the Datagrid is displayed)
		/// </summary>
		private string maxWidth = null;
		public string MaxWidth 
		{
			get { return maxWidth; }
			set { maxWidth = value; }
		}
		/// <summary>
		/// CSS class to use for the Datagrid header
		/// </summary>
		private string dgHeaderCss = null;
		public string DgHeaderCss 
		{
			get { return dgHeaderCss; }
			set { dgHeaderCss = value; }
		}
		/// <summary>
		/// CSS class to use for the Datagrid items
		/// </summary>
		public string DatagridCss 
		{
			get { return dgCss; }
			set { dgCss = value; }
		}
		private string dgCss = null;
		/// <summary>
		/// CSS class to use for the Datagrid alternate items
		/// </summary>
		private string dgAltCss = null;
		public string DgAltCss 
		{
			get { return dgAltCss; }
			set { dgAltCss = value; }
		}
		/// <summary>
		/// CSS class to use for the embedding panel
		/// </summary>
		private string css = null;
		public string Css 
		{
			get { return css; }
			set { css = value; }
		}

		/// <summary>
		/// ID of the control that builds the SecurityToken.
		/// </summary>
		private string loginID = null;
		public string LoginID 
		{
			get { return loginID; }
			set { loginID = value; }
		}

		/// <summary>
		/// ID of the control that queries the Commit.
		/// </summary>
		private string sourceID = null;
		public string SourceID 
		{
			get { return sourceID; }
			set { sourceID = value; }
		}

		/// <summary>
		/// Time between two Prepare invocations by the TransactionCoordinator
		/// </summary>
		private int timeout = 60;
		public string Timeout 
		{
			get { return timeout.ToString(); }
			set { timeout = Int32.Parse(value); }
		}

		/// <summary>
		/// Repository from which the Web service archive is deployed
		/// </summary>
		private string repositoryUrl;
		public string RepositoryURL 
		{
			get { return repositoryUrl; }
			set { repositoryUrl = value; }
		}

		/// <summary>
		/// Archive containing the Web service
		/// </summary>
		private string wsArchive;
		public string WSArchive 
		{
			get { return wsArchive; }
			set { wsArchive = value; }
		}

		/// <summary>
		/// Web service name
		/// </summary>
		private string wsName;
		protected System.Web.UI.WebControls.Label LabPrice;
	
		public string WSName 
		{
			get { return wsName; }
			set { wsName = value; }
		}

		/// <summary>
		/// Logical location of the client
		/// </summary>
		private string location;
		public string Location 
		{
			get { return location; }
			set { location = value; }
		}

		/// <summary>
		public ShoppingBasket()
		{
			this.Init += new System.EventHandler(Page_Init);
		}

		/// <summary>
		/// Populates the DropDown article list and the DataGrid.
		/// Page_Load also registers for login and transactions.
		/// Use of Coordinator objects:
		/// - Heap class: Get, Put
		/// - Coordinator abstract class: Subscribe
		/// - CallbackCoordinator extends Coordinator: Subscribe, Notify
		/// - TransactionCoordinator extends Coordinator: Subscribe, Commit
		/// - Callback interface: Notify (not used by ShoppingBasket)
		/// - Transaction interface: Prepare and Commit
		/// Object life:
		/// - Login: Session
		/// - Others: Request
		/// </summary>
		private void Page_Load(object sender, System.EventArgs e)
		{
			if (!IsPostBack) 
			{
				if (titleCss != null)
					title.CssClass = titleCss;
				if (statusCss != null)
					labStatus.CssClass = statusCss;
				if (copyrightCss != null)
					copyright.CssClass = copyrightCss;
				if (dgHeaderCss != null)
					dgShopping.HeaderStyle.CssClass = dgHeaderCss;
				if (dgCss != null)
					dgShopping.ItemStyle.CssClass = dgCss;
				if (dgAltCss != null)
					dgShopping.AlternatingItemStyle.CssClass = dgAltCss;
				if (css != null)
					ShoppingPanel.CssClass = css;
				ArrayList al = new ArrayList();
				for (int i = 1; i < 6; ++i)
					al.Add("Article " + i);
				ddArticle.DataSource = al;
				ddArticle.DataBind();
				ddArticle.SelectedIndex = 0;
				price.Text = "$1";
				Hashtable ht = (Hashtable)PageBoxGrid.Heap.Get(ID, Session.SessionID);
				if (ht != null) 
				{
					dgShopping.DataSource = ht.Values;
					dgShopping.DataBind();
					if (maxWidth != null)
						ShoppingPanel.Width = Unit.Pixel(Convert.ToInt16(maxWidth));
				}
				else 
				{
					if (minWidth != null)
						ShoppingPanel.Width = Unit.Pixel(Convert.ToInt16(minWidth));
					dgShopping.DataSource = new ArrayList();
					dgShopping.DataBind();
				}
				PageBoxGrid.TransactionCoordinator.Subscribe(sourceID, Session.SessionID, ID,
					this, timeout);
			}
			else 
			{
				if (PageBoxGrid.Heap.Get(ID, Session.SessionID) == null) 
				{
					if (minWidth != null)
						ShoppingPanel.Width = Unit.Pixel(Convert.ToInt16(minWidth));
					dgShopping.DataSource = new ArrayList();
					dgShopping.DataBind();
				}
			}
		}

		private void Page_Init(object sender, EventArgs e)
		{
			//
			// CODEGEN: This call is required by the ASP.NET Web Form Designer.
			//
			InitializeComponent();
		}

		#region Web Form Designer generated code
		///		Required method for Designer support - do not modify
		///		the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.ddArticle.SelectedIndexChanged += new System.EventHandler(this.OnArticleChanged);
			this.ButAdd.Click += new System.EventHandler(this.OnAdd);
			this.dgShopping.ItemCommand += new System.Web.UI.WebControls.DataGridCommandEventHandler(this.OnDelete);
			this.Load += new System.EventHandler(this.Page_Load);

		}
		#endregion

		/// <summary>
		/// Called when the user changes its selection on the Dropdown list
		/// </summary>
		private void OnArticleChanged(object sender, System.EventArgs e)
		{
			string[] flds = ddArticle.SelectedItem.Text.Split(new char[] { ' ' });
			price.Text = "$" + Int32.Parse(flds[1]);
			labStatus.Text = "";
		}

		/// <summary>
		/// Called when the user clicks on the Add button
		/// </summary>
		private void OnAdd(object sender, System.EventArgs e)
		{
			Hashtable ht = (Hashtable)PageBoxGrid.Heap.Get(ID, Session.SessionID);
			if (ht == null) 
			{
				ht = new Hashtable();
				if (maxWidth != null)
					ShoppingPanel.Width = Unit.Pixel(Convert.ToInt16(maxWidth));
			}
			CmdRow cr = null;
			try 
			{
				if (ht.Contains(ddArticle.SelectedItem.Text)) 
				{
					cr = (CmdRow)ht[ddArticle.SelectedItem.Text];
					cr.qty += Int32.Parse(TxtQty.Text);
					ht[ddArticle.SelectedItem.Text] = cr;
				} 
				else 
				{
					string p = price.Text.Substring(1);
					cr = new CmdRow(ddArticle.SelectedItem.Text, Int32.Parse(p),
						Int32.Parse(TxtQty.Text));
					ht.Add(ddArticle.SelectedItem.Text, cr);
				}
			}
			catch(FormatException fe) 
			{
				labStatus.Text = "Set the quantity";
				return;
			}
			labStatus.Text = "";
			PageBoxGrid.Heap.Put(ID, Session.SessionID, ht);
			dgShopping.DataSource = ht.Values;
			dgShopping.DataBind();
		}

		/// <summary>
		/// Called when the user clicks on a Delete button of the Datagrid  
		/// </summary>
		private void OnDelete(object source, System.Web.UI.WebControls.DataGridCommandEventArgs e)
		{
			int sel = e.Item.ItemIndex;
			if (sel != -1) 
			{
				string article = (string)dgShopping.DataKeys[sel];
				Hashtable ht = (Hashtable)PageBoxGrid.Heap.Get(ID, Session.SessionID);
				if (ht != null) 
				{
					ht.Remove(article);
					if (ht.Count == 0) 
					{
						PageBoxGrid.Heap.Remove(ID, Session.SessionID);
						if (minWidth != null)
							ShoppingPanel.Width = Unit.Pixel(Convert.ToInt16(minWidth));
					}
					else
						PageBoxGrid.Heap.Put(ID, Session.SessionID, ht);
					labStatus.Text = "";
					dgShopping.DataSource = ht.Values;
					dgShopping.DataBind();
				}
			}
		}

		/// <summary>
		/// 2-phase commit. Method of PageBoxGrid.Transaction.
		/// No rollback needed. Prepare cleans up in case of failure.
		/// </summary>
		/// <param name="sourceID">ID of the transaction source</param>
		/// <param name="req">Sent by Commit issuer (not used)</param>
		/// <param name="resp">Returned to the Commit issuer. Used to set error message</param>
		/// <param name="request">HTTP request object of the source</param>
		/// <returns>True in case of success</returns>
		public bool Prepare(string sourceID, object req, ref object resp, HttpRequest request) 
		{
			Hashtable ht = (Hashtable)PageBoxGrid.Heap.Get(ID, Session.SessionID);
			if (ht == null)
				return true;	// Why complain when nothing to do?
			string securityToken = (string)PageBoxGrid.Heap.Get(loginID, Session.SessionID);
			if (securityToken == null) 
			{
				PageBoxGrid.Heap.Remove(ID, Session.SessionID);
				resp = "Authenticate before committing your command";
				return false;
			}
			int userNb = Int32.Parse(securityToken.Substring(securityToken.Length - 3));
			SmartProxy sp = new SmartProxy(repositoryUrl, wsArchive, wsName, request);
			string wsUrl = sp.GetUrl(userNb, location);
			if (wsUrl != null) 
			{
				ShoppingOrder so = new ShoppingOrder(wsUrl);
				try 
				{
					so.Discover();
					if (so.CheckAccount(securityToken)) 
					{
						PageBoxGrid.Heap.Put(ID + "P", Session.SessionID, so);
						return true;
					}
					else
						resp = "ShoppingOrder.CheckAccount failed";
				}
				catch(Exception e) 
				{
					resp = e;
				}
			} 
			else
				resp = "No " + wsName + " instance available";
			ShoppingPanel.Width = Unit.Pixel(Convert.ToInt16(minWidth));
			PageBoxGrid.Heap.Remove(ID, Session.SessionID);
			return false;
		}

		/// <summary>
		/// 2-phase commit. Method of PageBoxGrid.Transaction.
		/// </summary>
		/// <param name="sourceID">ID of the transaction source</param>
		/// <param name="req">Sent by Commit issuer (not used)</param>
		/// <param name="resp">Returned to the Commit issuer. Used to set error message</param>
		/// <param name="request">HTTP request object of the source</param>
		/// <returns>True in case of success</returns>
		public bool Commit(string sourceID, object req, ref object resp, HttpRequest request) 
		{
			Hashtable ht = (Hashtable)PageBoxGrid.Heap.Get(ID, Session.SessionID);
			if (ht == null)
				return true;	// Why complain when nothing to do?
			ShoppingOrder so = (ShoppingOrder)PageBoxGrid.Heap.Get(ID + "P", Session.SessionID);
			if (so == null) 
			{
				PageBoxGrid.Heap.Remove(ID, Session.SessionID);
				resp = "No proxy found";
				return false;
			}
			PageBoxGrid.Heap.Remove(ID + "P", Session.SessionID);
			string securityToken = (string)PageBoxGrid.Heap.Get(loginID, Session.SessionID);
			if (securityToken == null) 
			{
				PageBoxGrid.Heap.Remove(ID, Session.SessionID);
				resp = "Authenticate before committing you command";
				return false;
			}
			OrderRow[] lines = new OrderRow[ht.Count];
			IDictionaryEnumerator ide = ht.GetEnumerator();
			int i = 0;
			while(ide.MoveNext()) 
			{
				CmdRow cr = (CmdRow)ide.Value;
				lines[i ++] = cr.GetOrderRow();
			}
			bool rc = true;
			try 
			{
				so.Order(securityToken, lines);
			}
			catch(Exception e) 
			{
				resp = e;
				rc = false;
			}
			PageBoxGrid.Heap.Remove(ID, Session.SessionID);
			ShoppingPanel.Width = Unit.Pixel(Convert.ToInt16(minWidth));
			return rc;
		}

		/// <summary>
		/// Transaction rollback. Method of PageBoxGrid.Transaction.
		/// </summary>
		/// <param name="sourceID">ID of the transaction source</param>
		/// <param name="req">Sent by Rollback issuer (not used)</param>
		/// <param name="resp">Returned to the Rollback issuer (not used)</param>
		/// <param name="request">HTTP request object of the source</param>
		/// <returns>True in case of success</returns>
		public bool Rollback(string sourceID, object req, ref object resp, HttpRequest request) 
		{
			Hashtable ht = (Hashtable)PageBoxGrid.Heap.Get(ID, Session.SessionID);
			if (ht == null)
				return true;	// Why complain when nothing to do?
			PageBoxGrid.Heap.Remove(ID, Session.SessionID);
			ShoppingPanel.Width = Unit.Pixel(Convert.ToInt16(minWidth));
			return true;
		}
	}

	/// <summary>
	/// Command row in the Datagrid.
	/// </summary>
	public sealed class CmdRow 
	{
		/// <summary>
		/// Article name
		/// </summary>
		private string article;
		/// <summary>
		/// Price of an article
		/// </summary>
		private int price;
		/// <summary>
		/// Number of articles
		/// </summary>
		public int qty;

		/// <summary>
		/// Constructor. Sets member variables.
		/// </summary>
		/// <param name="a">Article name</param>
		/// <param name="p">Price of an article</param>
		/// <param name="q">Number of articles</param>
		public CmdRow(string a, int p, int q) 
		{
			article = a;
			price = p;
			qty = q;
		}

		/// <summary>
		/// Returns the article name
		/// </summary>
		public string Article 
		{
			get { return article; }
		}

		/// <summary>
		/// Returns the row price = number of articles * article price
		/// </summary>
		public string Price 
		{
			get { return "$" + (price * qty); }
		}

		/// <summary>
		/// Returns the number of articles
		/// </summary>
		public string Quantity 
		{
			get { return qty.ToString(); }
		}

		public OrderRow GetOrderRow() 
		{
			OrderRow or = new OrderRow();
			or.article = article;
			or.price = price;
			or.qty = qty;
			return or;
		}
	}
}
